from sqlalchemy import *
from migrate import *
import migrate.changeset
from dictionaries.versions import views_helper
import re

def getDiscountsautoTables(meta):
    discountsauto1 = Table('discountsauto1', meta, autoload=True)
    discountsauto2 = Table('discountsauto2', meta, autoload=True)
    return [discountsauto1, discountsauto2]

class Dictionaries(object):
    def __init__(self, meta):
        self.tmccatalog = Table('tmccatalog', meta,
            Column('tmccode', String(100), nullable=False, primary_key=True),
            Column('catalogid', Integer, nullable=False, primary_key=True, autoincrement=False),
            )
        self.catalog = Table('catalog', meta,
            Column('catalogid', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('catalogname', String(100), nullable=True),
            )

def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    for tbl in getDiscountsautoTables(meta):
        catalogid = Column('catalogid', Integer, nullable=True, default=0)
        catalogquantbeg = Column('catalogquantbeg', Numeric(17, 3), nullable=True, default=0)
        catalogquantend = Column('catalogquantend', Numeric(17, 3), nullable=True, default=0)
        catalogsumbeg = Column('catalogsumbeg', Numeric(15, 2), nullable=True, default=0)
        catalogsumend = Column('catalogsumend', Numeric(15, 2), nullable=True, default=0)
        
        catalogid.create(tbl)
        catalogquantbeg.create(tbl)
        catalogquantend.create(tbl)
        catalogsumbeg.create(tbl)
        catalogsumend.create(tbl)
    views_helper.update_dic_view(migrate_engine, 'discountsauto')
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.create()
        basename = table.name[:-1]
        conn = migrate_engine.connect()
        conn.execute('create view %s as select * from %s' % (basename, basename + '1'))

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    # Operations to reverse the above upgrade go here.
    conn = migrate_engine.connect()
    tblname = views_helper.get_table_name(migrate_engine,'discountsauto')
    for tbl in getDiscountsautoTables(meta):
        tbl.c.catalogid.drop()
        tbl.c.catalogquantbeg.drop()
        tbl.c.catalogquantend.drop()
        tbl.c.catalogsumbeg.drop()
        tbl.c.catalogsumend.drop()
    views_helper.update_dic_view(migrate_engine, 'discountsauto',tblname)
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.drop()
        basename = table.name[:-1]
        conn.execute('drop view %s' % (basename))
