from sqlalchemy import *
from migrate import *
from dictionaries.versions import views_helper

table_name = "actionpanelitem"
columns = ["actionpanelcode", "row", "column"]
constraint_name = views_helper.make_unique_constraint_name(table_name, columns)


def remove_collisions(migrate_engine, _table_name):
    with migrate_engine.begin() as conn:
        '''
        В MySQL нету константы для самого большого целого числа и OFFSET нельзя использовать без LIMIT
        Поэтому чтобы LIMIT не накладывал ограничений на выборку в нём используется число
        MAX_INT = 18446744073709551615 как самое большое целое доступное в MySQL
        '''
        MAX_INT = 18446744073709551615
        conn.execute(
            text(
                "DELETE `{0}` "
                "FROM `{0}` "
                "INNER JOIN  "
                "   (SELECT `actionpanelitemcode`, `actionpanelcode`, `row`, `column` "
                "   FROM `{0}` "
                "   WHERE (`actionpanelcode`, `row`, `column`) "
                "   IN ("
                "       (SELECT `actionpanelcode`, `row`,`column` "
                "       FROM `{0}` "
                "       GROUP BY `actionpanelcode`, `row`, `column` "
                "   HAVING COUNT(*) > 1)"
                "   ) "
                "   LIMIT {1} OFFSET 1) as `to_del` "
                "ON "
                "   `{0}`.`actionpanelitemcode` = `to_del`.`actionpanelitemcode`;".format(
                    _table_name, MAX_INT
                )
            )
        )



def upgrade(migrate_engine):
    '''
    Поскольку в таблице уже могут содержаться данные нарушающие ограничение необходимо
    их удалить и оставить только те элементы которые отображаются на данный момент
    '''
    meta = MetaData(bind=migrate_engine)
    for tbl in views_helper.getTables(meta, table_name):
        remove_collisions(migrate_engine, tbl.name)
    views_helper.create_unique_constraint(
        migrate_engine,
        table_name,
        constraint_name,
        columns
    )


def downgrade(migrate_engine):
    views_helper.drop_unique_constraint(migrate_engine, table_name, constraint_name)
